@@ -1,18 +1,23 @@
 use bytes::{Bytes, BytesMut};
-use futures_util::{SinkExt, StreamExt};
-use tokio::sync::{mpsc, oneshot};
-use tokio_tungstenite::tungstenite;
+use futures_util::{Future, SinkExt, StreamExt};
+use tokio::{
+    sync::{mpsc, oneshot},
+    task::JoinHandle,
+};
+use tokio_tungstenite::tungstenite::{self, error::Error as SocketError};
 use tracing::debug;
 
-use super::error::*;
+use super::error::{ClientError, ClientResult};
 use crate::DecodeBodyError;
 
 type SenderChanWithReq<Req> = (Req, oneshot::Sender<Result<(), ClientError>>);
 type WebSocket =
     tokio_tungstenite::WebSocketStream<tokio_tungstenite::MaybeTlsStream<tokio::net::TcpStream>>;
 
-/// A websocket, wrapped for ease of use with protobuf messages.
-#[derive(Debug, Clone)]
+// This does not implement "close-on-drop" since socket instances may be sent across threads
+// by the user. This is done to prevent user mistakes.
+/// A hRPC socket.
+#[derive(Debug)]
 pub struct Socket<Req, Resp>
 where
     Req: prost::Message,
@@ -130,36 +135,39 @@ where
 
     /// Receive a message from the socket.
     ///
-    /// Returns a connection closed error if the socket is closed.
+    /// ## Errors
+    /// - Returns [`SocketError::ConnectionClosed`] if the socket is closed normally.
+    /// - Returns [`SocketError::AlreadyClosed`] if the socket is already closed.
+    ///
+    /// ## Notes
     /// This will block until getting a message if the socket is not closed.
     pub async fn receive_message(&self) -> ClientResult<Resp> {
         if self.is_closed() {
-            Err(ClientError::SocketError(
-                tungstenite::Error::ConnectionClosed,
-            ))
+            Err(ClientError::SocketError(SocketError::ConnectionClosed))
         } else {
             self.rx
                 .recv_async()
                 .await
-                .unwrap_or(Err(ClientError::SocketError(
-                    tungstenite::Error::ConnectionClosed,
-                )))
+                .unwrap_or(Err(ClientError::SocketError(SocketError::ConnectionClosed)))
         }
     }
 
     /// Send a message over the socket.
     ///
+    /// ## Errors
+    /// - Returns [`SocketError::ConnectionClosed`] if the socket is closed normally.
+    /// - Returns [`SocketError::AlreadyClosed`] if the socket is already closed.
+    ///
+    /// ## Notes
     /// This will block if the inner send buffer is filled.
     pub async fn send_message(&self, req: Req) -> ClientResult<()> {
         let (req_tx, req_rx) = oneshot::channel();
         if self.is_closed() || self.tx.send((req, req_tx)).await.is_err() {
-            Err(ClientError::SocketError(
-                tungstenite::Error::ConnectionClosed,
-            ))
+            Err(ClientError::SocketError(SocketError::ConnectionClosed))
         } else {
-            req_rx.await.unwrap_or(Err(ClientError::SocketError(
-                tungstenite::Error::ConnectionClosed,
-            )))
+            req_rx
+                .await
+                .unwrap_or(Err(ClientError::SocketError(SocketError::ConnectionClosed)))
         }
     }
 
@@ -173,4 +181,53 @@ where
         // We don't care about the error, it's closed either way
         let _ = self.close_chan.send(()).await;
     }
+
+    /// Spawns a parallel task that processes a socket.
+    pub fn spawn_task<T, Handler, HandlerFut>(
+        &self,
+        f: Handler,
+    ) -> JoinHandle<Result<T, ClientError>>
+    where
+        Handler: FnOnce(Self) -> HandlerFut + 'static,
+        HandlerFut: Future<Output = Result<T, ClientError>> + Send + 'static,
+        T: Send + 'static,
+    {
+        let sock = self.clone();
+        let fut = f(sock);
+        tokio::spawn(fut)
+    }
+
+    /// Spawns a parallel task that processes response messages and produces
+    /// request messages.
+    pub fn spawn_process_task<ProcessFn, ProcessFut>(
+        &self,
+        f: ProcessFn,
+    ) -> JoinHandle<Result<(), ClientError>>
+    where
+        ProcessFn: for<'a> Fn(&'a Self, Resp) -> ProcessFut + Send + Sync + 'static,
+        ProcessFut: Future<Output = Result<Req, ClientError>> + Send,
+    {
+        let sock = self.clone();
+        tokio::spawn(async move {
+            loop {
+                let req = sock.receive_message().await?;
+                let resp = f(&sock, req).await?;
+                sock.send_message(resp).await?;
+            }
+        })
+    }
+}
+
+impl<Req, Resp> Clone for Socket<Req, Resp>
+where
+    Req: prost::Message + 'static,
+    Resp: prost::Message + Default + 'static,
+{
+    fn clone(&self) -> Self {
+        Self {
+            close_chan: self.close_chan.clone(),
+            rx: self.rx.clone(),
+            tx: self.tx.clone(),
+        }
+    }
 }
