@@ -1,18 +1,26 @@
 use std::time::Duration;
 
 use bytes::BytesMut;
-use tokio::sync::{mpsc, oneshot};
+use futures_util::Future;
+use tokio::{
+    sync::{mpsc, oneshot},
+    task::JoinHandle,
+};
 use tokio_tungstenite::tungstenite::Error as SocketError;
 use tracing::{debug, error};
 
+use crate::DecodeBodyError;
+
 use super::{
-    prelude::ServerError,
+    error::ServerError,
     ws::{WebSocket, WsMessage},
 };
 
 type SenderChanWithReq<Resp> = (Resp, oneshot::Sender<Result<(), ServerError>>);
 
-/// A web socket.
+// This does not implement "close-on-drop" since socket instances may be sent across threads
+// by the user. This is done to prevent user mistakes.
+/// A hRPC socket.
 #[derive(Debug)]
 pub struct Socket<Req, Resp>
 where
@@ -29,7 +37,7 @@ where
     Req: prost::Message + Default + 'static,
     Resp: prost::Message + 'static,
 {
-    pub fn new(mut ws: WebSocket) -> Self {
+    pub(crate) fn new(mut ws: WebSocket) -> Self {
         let (recv_msg_tx, recv_msg_rx) = flume::bounded(64);
         let (send_msg_tx, mut send_msg_rx): (
             mpsc::Sender<SenderChanWithReq<Resp>>,
@@ -47,7 +55,8 @@ where
                             Ok(msg) => {
                                 match msg {
                                     WsMessage::Binary(data) => {
-                                        Req::decode(data.as_slice()).map_err(ServerError::MessageDecode)
+                                        Req::decode(data.as_slice())
+                                            .map_err(|err| ServerError::DecodeBodyError(DecodeBodyError::InvalidProtoMessage(err)))
                                     }
                                     WsMessage::Ping(data) => {
                                         if let Err(err) = ws.send(WsMessage::Pong(data)).await {
@@ -125,7 +134,11 @@ where
 
     /// Receive a message from the socket.
     ///
-    /// Returns `Err(SocketError::ClosedNormally)` if the socket is closed normally.
+    /// ## Errors
+    /// - Returns [`SocketError::ConnectionClosed`] if the socket is closed normally.
+    /// - Returns [`SocketError::AlreadyClosed`] if the socket is already closed.
+    ///
+    /// ## Notes
     /// This will block until getting a message if the socket is not closed.
     pub async fn receive_message(&self) -> Result<Req, ServerError> {
         if self.is_closed() {
@@ -140,6 +153,11 @@ where
 
     /// Send a message over the socket.
     ///
+    /// ## Errors
+    /// - Returns [`SocketError::ConnectionClosed`] if the socket is closed normally.
+    /// - Returns [`SocketError::AlreadyClosed`] if the socket is already closed.
+    ///
+    /// ## Notes
     /// This will block if the inner send buffer is filled.
     pub async fn send_message(&self, resp: Resp) -> Result<(), ServerError> {
         let (resp_tx, resp_rx) = oneshot::channel();
@@ -162,6 +180,41 @@ where
         // We don't care about the error, it's closed either way
         let _ = self.close_chan.send(()).await;
     }
+
+    /// Spawns a parallel task that processes a socket.
+    pub fn spawn_task<T, Handler, HandlerFut>(
+        &self,
+        f: Handler,
+    ) -> JoinHandle<Result<T, ServerError>>
+    where
+        Handler: FnOnce(Self) -> HandlerFut + 'static,
+        HandlerFut: Future<Output = Result<T, ServerError>> + Send + 'static,
+        T: Send + 'static,
+    {
+        let sock = self.clone();
+        let fut = f(sock);
+        tokio::spawn(fut)
+    }
+
+    /// Spawns a parallel task that processes request messages and produces
+    /// response messages.
+    pub fn spawn_process_task<ProcessFn, ProcessFut>(
+        &self,
+        f: ProcessFn,
+    ) -> JoinHandle<Result<(), ServerError>>
+    where
+        ProcessFn: for<'a> Fn(&'a Self, Req) -> ProcessFut + Send + Sync + 'static,
+        ProcessFut: Future<Output = Result<Resp, ServerError>> + Send,
+    {
+        let sock = self.clone();
+        tokio::spawn(async move {
+            loop {
+                let req = sock.receive_message().await?;
+                let resp = f(&sock, req).await?;
+                sock.send_message(resp).await?;
+            }
+        })
+    }
 }
 
 impl<Req, Resp> Clone for Socket<Req, Resp>
